// SelectionPersistence v0.1
function SelectionPersistence($grid) {
	var _selectedRows = {}, _restoreEnabled = false;
	// preliminary checks for dependencies
	if ($grid.element.data("igGridSelection") === undefined) {
		throw new Error("igGridSelection feature is not defined, but it's required. Please enable the selection feature.");
	}
	if ($grid.options.primaryKey === null) {
		throw new Error("igGrid.primaryKey should be defined in order for the SelectionPersistence to work properly");
	}
	
	function enableSelectionPersistence() {
		enableSelectionFilteringPersistence();
		enableSelectionPagingPersistence();
		enableSelectionSortingPersistence();
	}
	
	function disableSelectionPersistence() {
		disableSelectionPagingPersistence();
		disableSelectionFilteringPersistence();
		disableSelectionSortingPersistence();
	}
	
	function disableSelectionPagingPersistence() {
		$(document).off("iggridpagingpageindexchanging", "#" + $grid.element.attr("id"), saveSelection);
		$(document).off("iggridpagingpagesizechanging", "#" + $grid.element.attr("id"), saveSelection);
		var $rowSelectors = $grid.element.data("igGridRowSelectors");
		if ($rowSelectors !== undefined) {
			$rowSelectors.grid.element.bind('iggridpagingpageindexchanging', $rowSelectors._pageIndexChangingHandler);
			$rowSelectors.grid.element.bind('iggridpagingpageindexchanged', $rowSelectors._pageIndexChangedHandler);
		}
	}

	function disableSelectionSortingPersistence() {
		$(document).off("iggridsortingcolumnsorting", "#" + $grid.element.attr("id"), saveSelection);
	}
	
	function disableSelectionFilteringPersistence() {
		$(document).off("iggridfilteringdatafiltering", "#" + $grid.element.attr("id"), saveSelection);
	}

	function enableSelectionSortingPersistence() {
		if ($grid.element.data("igGridSorting") === undefined) {
			throw new Error("igGridSorting feature is not defined");
		}
		_selectedRows = {}
		bindRowsRendered();
		$(document).on("iggridsortingcolumnsorting", "#" + $grid.element.attr("id"), saveSelection);
	}
	
	function enableSelectionPagingPersistence() {
		if ($grid.element.data("igGridPaging") === undefined) {
			throw new Error("igGridPaging feature is not defined");
		}
		var $rowSelectors = $grid.element.data("igGridRowSelectors");
		if ($rowSelectors !== undefined) {
			$rowSelectors.grid.element.unbind('iggridpagingpageindexchanging', $rowSelectors._pageIndexChangingHandler);
			$rowSelectors.grid.element.unbind('iggridpagingpageindexchanged', $rowSelectors._pageIndexChangedHandler);
		}

		_selectedRows = {}
		bindRowsRendered();
		$(document).on("iggridpagingpageindexchanging", "#" + $grid.element.attr("id"), saveSelection);
		$(document).on("iggridpagingpagesizechanging", "#" + $grid.element.attr("id"), saveSelection);
	}
	
	function enableSelectionFilteringPersistence() {
		if ($grid.element.data("igGridFiltering") === undefined) {
			throw new Error("igGridFiltering feature is not defined");
		}
		_selectedRows = {}
		bindRowsRendered();
		$(document).on("iggridfilteringdatafiltering", "#" + $grid.element.attr("id"), saveSelection);
	}
	
	function bindRowsRendered() {
		if (_restoreEnabled === false) {
			$(document).on("iggridrowsrendered", "#" + $grid.element.attr("id"), function (evt, ui) {
				restoreSelection();
			});
		}
		_restoreEnabled = true;
	}
	
	function restoreSelection() {
		var row, ix, tmp = [];
		for(row in _selectedRows) {
			ix = $grid.element.find("tr[data-id='" + row + "']").index();
			if (ix >= 0) {
				$grid.element.data("igGridSelection").selectRow(ix);
				tmp.push(row);
			}
		}
		for (var i = 0; i < tmp.length; i++) {
			delete _selectedRows[tmp[i]];
		}
	}
	
	function saveSelection() {
		var currentSelection = $grid.element.data("igGridSelection").selectedRows();
		for (var i = 0; i < currentSelection.length; i++) {
			_selectedRows[currentSelection[i].id] = true;
		}
	}

	function clearSelection() {
		$grid.element.data("igGridSelection").clearSelection();
		_selectedRows = {};
	}
	
	function getSelectedRows() {
		saveSelection();
		return _selectedRows;
	}
	// public API
	return {
		selectedRows: getSelectedRows,
		enable: enableSelectionPersistence,
		enablePaging: enableSelectionPagingPersistence,
		enableFiltering: enableSelectionFilteringPersistence,
		clearSelection: clearSelection,
		disable: disableSelectionPersistence
	}
}
